      subroutine fcn(y,t,dy,dt,neq,flux,nflux) 
      implicit double precision (a-h,o-z) 
      dimension dy(neq),y(neq) 
      dimension flux(nflux) 

c define mitochondrial variables and parameters
      double precision fm 
      double precision Nai, n, F, Cai, R, Jnc, Nam
      double precision pH, Gh, Cm, Temp 
      double precision Kres, RHOr, r1, r2 
      double precision r3, Ra, Rb 
      double precision Rc1, Rc2, Psib, g, NADm 
      double precision fp, ADPi, ATPi 
      double precision Cam, ADPm, Psi 
      double precision Jhl, ATPm, dp, ADP3m, ADP4m, ATP4i 
      double precision ADP3i, Ares, ATP4m 
      double precision X1, X2, X3, X4, Juni, Iuni
      double precision Y1, Y2, Y3, Y4, Y5, Jhr, JO 
      double precision Q1, Q2, Q3, Q4, Q5, Jp 
      double precision Jhf 
      double precision V1, V2, V3, Jant 
      double precision NADrat,j 
      double precision Jnh, Jpic, Pimito
      double precision Jkh, Ke, Kx
      double precision Jop, Vadp, Kadp, Picyt, Vatp, Katp, Kpi
      double precision KHi, KHm, KNam, KNai, KmPicyt, KmPim, KiOHm, KiOHcyt
      integer*4 i 
 
c Mitochondrial Ca2+ Handling Parameters 
      parameter (Nai=5.0, F=96484.6) 
      parameter (R=8314.0) 
      parameter (Volume=10.336d-18)
 
c Model Parameters 
      parameter (Temp=310.0) 

c Respiration Driven H+ Pump Parameters 
      parameter (Kres=1.35d18, RHOr=0.4, r1=2.077d-18, r2=1.728d-9) 
      parameter (r3=1.059d-26, Ra=6.394d-10, Rb=1.762d-13) 
      parameter (Rc1=2.656d-19, Rc2=8.632d-27, Psib=50, g=0.85, NADm=6) 
 
c ANT paramters 
      parameter (fp=0.5,ADPi=1.0,ATPi=4.0) 
      
c     Define variables 
      acoa = y(1)  
      cit = y(2)  
      aisoc = y(3) 
      akg = y(4) 
      scoa = y(5) 
      suc = y(6) 
      fum = y(7) 
      amal = y(8) 
      oaa = y(9) 
      asp = y(10) 
      glu = y(11) 
      Cam = y(12) 
      amg = y(13) 
      coa = y(14) 
      ADPm = y(15) 
      anad = y(16) 
      anadh = y(17) 
      ah = y(18) 
      ATPm = y(19) 
      psi = y(20) 
      afadh2 = y(21) 
      Cai = y(22) 
      Nam = y(23) 
      Pimito = y(24)
      
c     Define fluxes 
 
c     isocitrate dehydrogenase (fidh) 
      a_1 = 1.0 + ah/(8.10d-8) + (5.98d-8)/ah 
      a_2 = 1.0/(1.0+ADPm/0.062)/(1.0+Cam/0.141) 
      a_3 = ((0.923/anad)**1.42)*(1.0+anadh/0.230) 
      fidh = (0.0008488)/(a_1+((1.52/aisoc)**0.627)*a_2 + 
     +       a_3 + ((4.07/aisoc)**0.627)*a_2*a_3) 
 
c     citrate synthase (fcs) 
      fcs = (0.0005072)*acoa*oaa/(acoa*oaa + 0.0126*oaa + 
     +      6.30d-4*acoa*(1.0+acoa/0.037) + 0.081*6.30d-4) 
 
c     input flux (fin) 
      fin = fcs 
 
c     aconitase hydratase (faco) 
      faco = (0.003989)*(cit - (1.0/0.45)*aisoc) 
 
c     alpha-ketoglutarate dehydrogenase (fkgdhc) 
      a_1 = (24.70/akg)/((1.0+amg/0.157)*(1.0+Cam/0.0013)) 
      a_2 = ((31.04/anad)**0.423)/((1.0+amg/0.157)*(1.0+Cam/0.0013)) 
      fkgdhc = (0.01771)/(1.0+a_1+a_2) 
 
c     aspartate aminotransferase (faat) 
      faat = (0.6498)*(oaa*glu - (1.0/6.6)*akg*asp) 
 
c     succinyl-CoA ligase (fsl) 
      fsl = (0.001277)*(scoa - 1.0/3.115*suc*coa) 
 
c     succinate dehydrogenase (fsdh) 
      fsdh = (0.0008961)/(1.0+(0.3/suc)*(1.0+fum/2.6)) 
 
C     fumarate hydratase (ffh) 
      ffh = (0.001277)*(fum - 1.0/4.4*amal) 
 
c     malate dehydrogenase (fmdh) 
      a_1 = 1.0/(1.0+ah/1.13d-9+ah*ah/(1.13d-9*0.0267)) + 0.0399 
      a_2 = (1.0/(1.0+6.68d-14/ah+(6.68d-14*5.62d-10)/(ah*ah)))**2 
      a_3 = a_1*a_2 
      a_1 = (1.29/amal)*(1.0+oaa/0.00314) 
      fmdh = (0.09672)*a_3/(1.0+a_1+(0.179/anad)+a_1*(0.059/anad)) 

c     Mitofluxes 

c     rescale concentrations for use in mitochondrial model

      Cm = 0.000812
      aHe = 10**-7.1				

      fm = 0.10 		! 0.03 Corkeyetal1996 0.35mm/12.7mm=0.03 superceeds M-K 1982 reference from same group. 
      NADrat = y(17)/y(16) 

c functions 
         pHe = -dlog10(aHe)
         pHm = -dlog10(aH)
	 pH = pHe - pHm
         dp=Psi-(((2.303*R*Temp)/F)*pH)
	 Gh=1.867d-6
         Jhl=Gh*dp 
         ADP3m=0.45*0.8*ADPm 
         ADP4m=0.05*ATPm
         ATP4i=0.05*ATPi 
         ATP4m=0.05*ATPm
         ADP3i= 0.45*ADPi 
         Ares= (R*Temp/F)*(dlog(Kres*(NADrat**0.5))) 

c uniporter 
        Vrtf = -Psi*F/(R*Temp)
	vrtf2=vrtf*F
        PCa=2.159
	 Juni = -(PCa*2.d0*vrtf*(0.2*(Cam)*dexp(2.d0*vrtf)
     &        -(Cai)*0.341d0)
     &          /  (dexp(2.d0*vrtf)-1.d0))

c Na/Ca exchanger (2/26/06)
        akna = 8.0        !mM 9.4 WingroveGunter 8.2 mM Paucek 2004
        akca = 0.008     !mM 4.54 WingroveGunter 0.002-0.00009 mM Paucek 2004 0.0013 uM Crompton et al 1977

        vmax = 0.01863

        ak1 = dexp(0.5*Psi*F/(R*Temp))
        ak2 = dexp(-0.5*Psi*F/(R*Temp))

        y1 = ak1*(Nai**3.0)*Cam/((akna**3.0)*akca)
        y2 = ak2*(Nam**3.0)*Cai/((akna**3.0)*akca)
        y3 = ((Nai/akna)**3.0)+Cam/akca+((Nai**3.0)*Cam)/((akna**3.0)*akca) 
        y4 = ((Nam/akna)**3.0)+Cai/akca+((Nam**3.0)*Cai)/((akna**3.0)*akca)

        Jnc = vmax*(y1-y2)/(1.0+y3+y4) 


c Na/H exchanger
        KHi = 0.0001d-6				!M, fit to Kapus 1989
	KHm = KHi
	KNam = 20.0				!mM, from Kapus 1989
	KNai = KNam				

	VNaH = 762.0*((aH)**1.0)/(((aH)**1.0) + (0.0757*1.d-6)**1.0)
	y1 = VNaH*aHe*Nam/(KHi*KNam)
	y2 = VNaH*ah*Nai/(KHm*KNai)
	y3 = aHe/KHi+Nam/KNam+aHe*Nam/(KHi*KNam)+ah/KHm+Nai/KNai+ah*Nai/(KHm*KNai)
        Jnh = (y1-y2)/(1.0+y3)


c K/H antiporter
      	Xkh = 5.1344d2
     	Ke = 140			!extramitochondrial potassium 140 mM
     	Kx = 111.0			!mitochondrial potassium 180 mM Gruweleta;l 1999                        
      	Jkh = Xkh*(Ke*ah-Kx*aHe)      !scaling Xkh volume (Leem1999)

c Phophate carrier			!(Pi/Pi) Pi/OH counter-transport
	
c	VPic = 45.36
	VPic = 21.21
        KmPicyt = 1.75			!(mM) Stappen-Kramer 1994
        KiOHm = 206.42d-9			!(M) Stappen-Kramer 1994
        Picyt = 2.78			!(mM) Corkey-Williamson 1985
c        Picyt = 1.3			!(mM) Corkey-Williamson 1985
	KmPim = 11.0			!(mM) Stappen-Kramer 1994
	KiOHcyt = 32.84d-9		!(M) Stappen-Kramer 1994
	OHm = 1.d-14/aHe
	OHcyt = 1.d-14/ah
	
	y1 = Vpic*Picyt*OHm/(KmPicyt*KiOHm)
	y2 = Vpic*Pimito*OHcyt/(KmPim*KiOHcyt)
        y3 = Picyt/KmPicyt+OHm/KiOHm+Picyt*OHm/(KmPicyt*KiOHm) 
        y4 = Pimito/KmPim+OHcyt/KiOHcyt+Pimito*OHcyt/(KmPim*KiOHcyt) 
 
	Jpic = (y1-y2)/(1.0+y3+y4)
	
c proton pump from respiration 
        Y1=(Ra*(10**(6*pH))*(dexp((F*Ares)/(R*Temp)))) 
        Y2=(Ra+Rb)*(dexp((g*6*F*Psi)/(R*Temp))) 
        Y3=(1+(r1* (dexp ((F*Ares)/(R*Temp)) )))* 
     +     (dexp((6*F*Psib)/(R*Temp))) 
        Y4=(r2 + (r3*(dexp((F*Ares)/(R*Temp)))))* 
     +     ( dexp((g*6*F*Psi)/(R*Temp))) 
         Jhr=0.003360*RHOr*((Y1-Y2)/(Y3+Y4)) 
 
c single oxygen atom consumption rate 
        Y1=(Ra*(10**(6*pH)) + Rc1*dexp(6*F*Psib/(R*Temp))) 
     +        *dexp((F*Ares)/(R*Temp)) 
	Y2=-Ra*dexp((g*6*F*Psi)/(R*Temp)) 
     +       +Rc2*dexp((F*Ares)/(R*Temp))*dexp((g*6*F*Psi)/(R*Temp)) 
	 JO=0.0002800*RHOr*(Y1+Y2)/(Y3+Y4) 
 
c Phosphorylation and New proton flux due to ATPase 
	Vadp=1.0		
	Kadp=.06		!10uM for [ADP] < 166uM, Tomashek 2004
	Vatp=0.08		!V3/V4 = 5, Berkich...LaNoue, 1991
	Katp=0.0322
	Kpi=.150  		!150-450uM

	Y1=Vadp*(ADPm/Kadp)*(Pimito/Kpi) - Vatp*(ATPm/Katp)
	Y2=(1+ADPm/Kadp)*(1+Pimito/Kpi)
	Y3=ATPm/Katp
	Jop=Y1/(Y2+Y3)	      
         
        fatp = 11.25d-4*Psi**8./(70.0**8.+Psi**8.)
	Jp = fatp * (1-dexp(-Cam/0.165d-3))*Jop				!165nM=K

	Jhf = 3.0*Jp

c ATP/ADP antiport flux 
        V1=((ATP4i)/(ADP3i))*((ADP3m)/(ATP4m))*(dexp((-F*Psi)/(R*Temp))) 
        V2= 1+ (((ATP4i)/(ADP3i))*(dexp((-fp*F*Psi)/(R*Temp)))) 
        V3=1+ ((ADP3m)/(ATP4m))  
         Jant=1.87d-3*((1-V1)/(V2*V3)) 
         
c     FADH2 net flux (ffadh2) 
      ffadh2 = fsdh - 4.0/3.0*JO 
      
c     NADH production flux (fnadhp) 
      fnadhp = fidh + fkgdhc + fmdh 
 
c     NADH consumption flux (fnadhc) 
      fnadhc = 2.0*JO 
 
c     NADH net flux (fnadh) 
      fnadh = fnadhp - fnadhc 
 
c     CO2 flux (fco2) 
      fco2 = fidh + fkgdhc  
 
 
c     Store fluxes 
      flux(1) = fcs 
      flux(2) = faco 
      flux(3) = fidh 
      flux(4) = fkgdhc 
      flux(5) = faat 
      flux(6) = fsl 
      flux(7) = fsdh 
      flux(8) = ffh 
      flux(9) = fmdh 
      flux(10) = fin 
      flux(11) = fnadh 
      flux(12) = ffadh2 
      flux(13) = fco2 
      flux(14) = fnadhp 
      flux(15) = fnadhc 
      flux(16) = Juni
      flux(17) = Jnc
      flux(18) = Jp
      flux(19) = Jhf
      flux(20) = Jhr
      flux(21) = Jnh
      flux(22) = Jpic
      flux(23) = Jkh
      flux(24) = JO
      
c     Define derivatives 
 
      dy(1) = fin - fcs 
      dy(2) = fcs - faco	 
      dy(3) = faco - fidh 
      dy(4) = fidh - fkgdhc + faat  
      dy(5) = fkgdhc - fsl  
      dy(6) = fsl - fsdh  
      dy(7) = fsdh - ffh 
      dy(8) = ffh - fmdh  
      dy(9) = fmdh - fcs - faat 
      dy(10) = 0.0 
      dy(11) = 0.0 
      dy(12) = fm * (Juni - Jnc)
      dy(13) = 0.0 
      dy(14) = 0.0 
      dy(15) = Jant - Jp 
      dy(16) = -fnadh
      dy(17) = fnadh 
      dy(18) = (Jnh + Jhf + Jhl - Jhr + Jpic - Jkh)*0.001*0.00001	!0.001 to convert mM to M and 0.001 for buffering
      dy(19) = Jp - Jant
      dy(20) = (1/cm) * (-(-Jhr + Jhf + Jant + Jhl + 2*Juni + Jnc)) 
      dy(21) = 0.0 
      dy(22) = 0.0 
      dy(23) = 3.0*Jnc - Jnh 
      dy(24) = Jpic - Jp
      return 
      end 
